#include <avr/io.h>
#include "i2cmaster.h"
#include <avr/signal.h> 
#include <avr/interrupt.h>
#include <inttypes.h>

#define Slave1  4    // device address of EEPROM 24C02, see datasheet

void uart_send_u8(uint8_t y);
void uart_send_u16(uint16_t x);
void twi__(void);

volatile uint8_t temp=1;


int main(void)
{
    DDRB  = 0xff;                              	// use all pins on port B for output 
    PORTB = 0xff;                              	// (active low LED's )
    
    UBRRL=12;								// Set baudrate 38400  (8MHz/ 16*(UBRR+1)=baud  =12	
    UCSRB|= _BV(RXEN)|_BV(TXEN);		// UART recieve on, UART Send on	
    UCSRB|= _BV(RXCIE);					// UART RX complete interrupt enable

	i2c_init();								// init I2C interface
    
	sei();
      
	twi__();  
	for(;;);	
}

//////////////////////////////////////////////////////////////////////////////////////////
// recieve from USART stopps the program untill all values arrived			
////////////////////////////////////////////////////////////////////////////////////////// 
SIGNAL(SIG_UART_RECV)
{       
	temp = UDR;        	// get the character
 }



//////////////////////////////////////////////////////////////////////////////////////////
// 	TWI software Master	
////////////////////////////////////////////////////////////////////////////////////////// 
void twi__()
{
	unsigned char ret;
	int data_=10;
	
	
	
	for(;;)									// write 0x75 to eeprom address Data  0x05 (Byte Write) 
	{
 
		ret = i2c_start((Slave1<<1)+I2C_WRITE);        	// set device address and write mode
		if(ret){							// failed to issue start condition, possibly no device found 
			i2c_stop(); 
			uart_send_u8(1);				// show error 
		}
		else {								// issuing start condition ok, device accessible 
			i2c_write(temp);                	// write data = 5
			i2c_stop();                        	// set stop conditon = release bus
			
			
			uint16_t i;
			for(i=0;i<50;i++) asm volatile ("nop");
			
			
			i2c_start_wait((Slave1<<1) +I2C_READ);      	// set device address and read mode
			data_ = i2c_readNak();                    			// read one byte
			i2c_stop();
			
			uart_send_u8(data_);
		} 
		
		uint16_t i;
		for(i=0;i<5000;i++) asm volatile ("nop");
	}
}




//////////////////////////////////////////////////////////////////////////////////////////
// USART sendet 8 bit										
////////////////////////////////////////////////////////////////////////////////////////// 
void uart_send_u8(uint8_t data)
{	
	UCSRB |= _BV(TXEN);				// TX aktivieren
	while(!(UCSRA & (1<<UDRE)));		// Warten, bis UDRE eins ist-->UDR ist leer	
	UDR=(uint8_t)data;		  			// data_to_send in das UDR Register schreiben

}
//////////////////////////////////////////////////////////////////////////////////////////
// USART sendet 16 bit										
////////////////////////////////////////////////////////////////////////////////////////// 
void uart_send_u16(uint16_t data_to_send)
{			
	uart_send_u8((uint8_t)(data_to_send & 0xFF));	// Low Wert ->PC
	uart_send_u8((uint8_t)(data_to_send >> 8)); 		// High Wert ->PC
}	
